	PROGRAM VGPEST3
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	                                                                      C
C	Program written: Dec.9, 1993                                          C 
C	Last Updated:	 Dec 22, 1993, 2003                                   C
C	Written by:	Rafael Munoz-Carpena				      C
C	Assistant Professor, Hydrology & Water Quality
C	University of Florida
C	IFAS-Tropical Research and Education Center
C	18905 SW 280 Street, Homestead, FL  33031-3314 
C	Office: 305 246-7001 x288       Suncom 478-7001 x288 
C	Fax:    305 246-7003            Suncom 478-7003 
C	http://carpena.ifas.ufl.edu/  ; email: carpena@mail.ifas.ufl.edu
C	                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	                                                                      C
C	This program is a module for estimating the 3 parameters (a,n,Wr) for C
C	the van Genuchten soil-water characteristic curve formula.bm) using   C
C	optimization  subroutine LMDIF1 from the math library MINPACK	      C
C	(Argonne National Laboratory).					      C
C	This Program reads the SWCC lab values, water content (W, cm3/cm3) vs C
C	suctions (h, -cm water) and calculates the parameters such as: 	      C
C                                                                             C
C	W_c(h) = Wr + (Ws-Wr)*[1+(a.h)^n]^(-m);	m=1 -1/n               (1)    C
C                                                                             C
C	and then calculates the unsaturated conductivity curve as:	      C
C           W_c(h)- Wr                                                        C
C	Se= ---------- 		                                       (2)    C
C            Ws - Wr                                                          C
C	Kr(Se)=Ku/Ks = Se^0.5 [1-(1-Se^(1/m))^m]^2		       (3)    C
C                                                                             C
C       where:								      C
C	W(h),W_c(h)  = 	Measured and predicted water contents                 C                    
C	Se(i,2)      = 	effective saturation (j=2) for each pressure step(j=1)C
C	SWCC(i,2)    = 	Water content (j=2)for each suction step(j=1)         C
C	Ws, Wr, Wr_c = 	Saturated and Residual water content (_c=predicted)   C                    
C	Ks (cm/h)    = 	Saturated conductivity.		        	      C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	       ***Original documaentation from LMDIF *****                                                                      C
C       FCN IS THE NAME OF THE USER-SUPPLIED SUBROUTINE WHICH
C         CALCULATES THE FUNCTIONS. FCN MUST BE DECLARED
C         IN AN EXTERNAL STATEMENT IN THE USER CALLING
C         PROGRAM, AND SHOULD BE WRITTEN AS FOLLOWS.
C
C         SUBROUTINE FCN(M,N,X,FVEC,IFLAG)
C         INTEGER M,N,IFLAG
C         DOUBLE PRECISION X(N),FVEC(M)
C         ----------
C         CALCULATE THE FUNCTIONS AT X AND
C         RETURN THIS VECTOR IN FVEC.
C         ----------
C         RETURN
C         END
C
C      IFLAG SHOULD NOT BE CHANGED BY FCN UNLESS THE USER WANTS
C	T TO TERMINATE EXECUTION OF LMDIF1.
C         IN THIS CASE SET IFLAG TO A NEGATIVE INTEGER.
C
C       MPAR IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF FUNCTIONS. [i.e. M= (no. nodes).(no.time steps)]
C
C       NPAR IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF VARIABLES. N MUST NOT EXCEED M. (i.e. N=4)
C
C       X IS AN ARRAY OF LENGTH N. ON INPUT X MUST CONTAIN
C         AN INITIAL ESTIMATE OF THE SOLUTION VECTOR. ON OUTPUT X
C         CONTAINS THE FINAL ESTIMATE OF THE SOLUTION VECTOR.
C
C       FVEC IS AN OUTPUT ARRAY OF LENGTH M WHICH CONTAINS
C         THE FUNCTIONS EVALUATED AT THE OUTPUT X.
C
C       TOL IS A NONNEGATIVE INPUT VARIABLE. TERMINATION OCCURS
C         WHEN THE ALGORITHM ESTIMATES EITHER THAT THE RELATIVE
C         ERROR IN THE SUM OF SQUARES IS AT MOST TOL OR THAT
C         THE RELATIVE ERROR BETWEEN X AND THE SOLUTION IS AT
C         MOST TOL.
C
C       INFO IS AN INTEGER OUTPUT VARIABLE. IF THE USER HAS
C         TERMINATED EXECUTION, INFO IS SET TO THE (NEGATIVE)
C         VALUE OF IFLAG. SEE DESCRIPTION OF FCN. OTHERWISE,
C         INFO IS SET AS FOLLOWS. (SEE CMESS SUBROUTINE)
C
C       IWA IS AN INTEGER WORK ARRAY OF LENGTH N.
C
C       WA IS A WORK ARRAY OF LENGTH LWA.
C
C       LWA IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN
C         MPAR*NPAR+5*NPAR+MPAR.
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	IMPLICIT DOUBLE PRECISION   (A-H,O-Z)
	PARAMETER(MAXM=20, NPAR=3, MLWA=95) 
     	INTEGER IWA(NPAR)
	CHARACTER*9 FILENM1

	common/datos/FILENM1

      	DIMENSION X(NPAR),FVEC(MAXM),WA(MLWA)
     	EXTERNAL FCN

	CALL BANNER
	CALL PREINPUT(MPAR,NPAR,LWA,TOL,X)
     	CALL LMDIF1(FCN,MPAR,NPAR,X,FVEC,TOL,INFO,IWA,WA,LWA,NFEV)
      	CALL CMESS(INFO,NFEV,NPAR,X)
        CALL KRESULTS(MPAR,X)

	STOP
	END


	SUBROUTINE BANNER
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C        To print program information and purpose                             C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	IMPLICIT DOUBLE PRECISION   (A-H,O-Z)

C-------Intro stuff---------------------------
	WRITE(*,*)' '
	WRITE(*,*)' '
	WRITE(*,*)' '
	WRITE(*,*)'              \      --    ----   ----    ----   ---  ---
     & ----'
	WRITE(*,*)'               |    /    /       /    /  /     /        /'
	WRITE(*,*)'               |   /    /  ---  /----   ----   ---     /'
	WRITE(*,*)'               |  /    |    /  /       /          /   /'
	WRITE(*,*)'               |/       ---   /       -----  ----    /'
	WRITE(*,*)' '
	WRITE(*,*)'          ************************************************'
	WRITE(*,*)'          *        VGPEST-Version 1.1, 15/Spet/97        *'
	WRITE(*,*)'          *>>>>>>3 PARAMETER VERSION (Wr, alpha, n)<<<<<<*'
	WRITE(*,*)'          *           Rafael Munoz-Carpena               *'
	WRITE(*,*)'          * U. of Florida (USA). e-mail:carpena@ufl.edu  *'
	WRITE(*,*)'          *                                              *'
	WRITE(*,*)'          ************************************************'
	WRITE(*,*)' '
	WRITE(*,*)' '
	WRITE(*,*)'----Press any letter and return to continue---------'
c	READ(*,*)
	WRITE(*,*)'                    INTRODUCTION'
	WRITE(*,*)'======================================================='
	WRITE(*,*)'This program estimates the parameters (a, n,Wr) for the van'
	WRITE(*,*)'Genuchten soil-water characteristic curve model.It uses the'
	WRITE(*,*)'Levenberg-Marquardt non-linear optimization algorithm
     & (subroutine'
	WRITE(*,*)'LMDIF1, MINPACK math library, Argonne National Laboratory).
     & The'
	WRITE(*,*)'program reads in the SWCC lab values, water content(cm3/cm3)
     & vs'
	WRITE(*,*)'pressure heads (-cm water), and fits the parameters such
     & as:'
	WRITE(*,*)' '
	WRITE(*,*)'    W_c(h) = Wr + (Ws-Wr)*[1+(a.h)^n]^(-m) ;m=1 -1/n  (1)'
	WRITE(*,*)' '
	WRITE(*,*)'It then calculates the unsaturated conductivity curve as:'
	WRITE(*,*)'           W_c(h)- Wr'
	WRITE(*,*)'    Se(h)= ---------- = [1+(a.h)^n]^m                 (2)'
	WRITE(*,*)'            Ws - Wr'                                                          
	WRITE(*,*)'    Kr(Se)=Ku/Ks = Se^0.5 [1-(1-Se^(1/m))^m]^2        (3)'
	WRITE(*,*)' '
	WRITE(*,*)'where:'
	WRITE(*,*)'W(h),W_c(h)= Soil-water characteristic curve (_c=predicted)'
	WRITE(*,*)'Se(h)       = Effective saturation for each pressure step'
	WRITE(*,*)'Ws, Wr      = Saturated and Residual water content'                   
	WRITE(*,*)'Ks(cm/h)    = Saturated conductivity'
	WRITE(*,*)'----Press any letter and return to continue---------'
c	READ(*,*)dummy

	RETURN
	END


	SUBROUTINE PREINPUT(MPAR,NPAR,LWA,TOL,X)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C        - Read values of SWCC and calculate SE, MPAR and LWA                 C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	IMPLICIT DOUBLE PRECISION   (A-H,O-Z)
	PARAMETER(MAXM=20)

	CHARACTER*35 DUMMY,TITLE
	CHARACTER*9 FILENM1

	common/datos/FILENM1

c	DIMENSION SE(MAXM,2),SWCC(MAXM,2),X(NPAR)
	DIMENSION SWCC(MAXM,2),X(NPAR)

c-------get file name from input line----------
c***UNIX/DOS selection***********
CDOS 	INARGS=NARGS()-1
CDOS 	IF (INARGS.EQ.1) THEN
CDOS 	CALL GETARG(1,FILENM1,FSTATUS)
CUNIX
	INARGS=IARGC()
CUNIX
	IF (INARGS.EQ.1) THEN
CUNIX
	  CALL GETARG(1,FILENM1)
c***End of UNIX/DOS selection***********
	 ELSE
          WRITE(*,*)
c          WRITE(*,105)
	  WRITE(*,110)
	  WRITE(*,120)
          WRITE(*,130)
	  WRITE(*,140)
          WRITE(*,150)
          WRITE(*,*)
          STOP
      	ENDIF
	OPEN(3,FILE=FILENM1,STATUS='OLD')
	
c-------Read data-----------------

	TOL=1.D-20
	READ(3,*)DUMMY
	TITLE=DUMMY
	READ(3,*)(X(I),I=1,NPAR)
	READ(3,*)VKS
	DO 10 I=1,20
		READ(3,*,END=20)(SWCC(I,J),J=1,2)
10	CONTINUE
20	NPOINT=I-1
c	WR=X(3)
	WS=SWCC(1,2)
c	WD=WS-WR
	WRITE(*,*)'                    INPUTS'
	WRITE(*,*)'======================================================='
	WRITE(*,*)' SAMPLE DATA SET: ',TITLE
	WRITE(*,*)' '
	WRITE(*,*)'  h(cm)   W(cm3/cm3)'
	WRITE(*,*)'----------------------'
	DO 30 I=1,NPOINT
c		SE(I,1)=SWCC(I,1)
c		SE(I,2)=(SWCC(I,2)-WR)/WD
		WRITE(*,100)(SWCC(I,J),J=1,2)
30	CONTINUE
	MPAR = NPOINT
	LWA=MPAR*NPAR+5*NPAR+MPAR
	WRITE(*,*)' '
	WRITE(*,*)'Ws=',WS
	WRITE(*,*)'Ks(cm/h)=',VKS
	WRITE(*,*)'Tol.=',TOL,' ;Initial guess (alpha,n,Wr)=',(x(i),i=1,npar)
	WRITE(*,*)' '
	WRITE(*,*)'----Press any letter and return to continue---------'
c	READ(*,*)DUMMY
	WRITE(*,*)' '
	WRITE(*,*)'                   RESULTS'
	WRITE(*,*)'======================================================='
	WRITE(*,*)' SAMPLE DATA SET: ',TITLE
	CLOSE(3)
100	FORMAT(F8.2,F10.6)	
c105	FORMAT('Name:    vgpest3')
c110	FORMAT(9x,'(Program to fit van Genuchten curve to experimental soil')
c120	FORMAT(10x,'and water experimental data using MINPACK optimization)')
110	FORMAT('********************  WARNING  *************************')
120	FORMAT('*                                                      *')
130	FORMAT('*  Program usage: vgpest3 filename (max 8 characters)  *')
140	FORMAT('*                                                      *')
150	FORMAT('********************************************************')
c130	FORMAT('Program usage:   vgpest3 filename (max 8 characters)')
c140	FORMAT('Version: 1.01 for Unix - September 1997')
c150	FORMAT('Author: R.Munoz-Carpena - ICIA')
	
	RETURN
	END


         SUBROUTINE FCN(MPAR,NPAR,X,FVEC,IFLAG)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C        Subroutine as defined in original MINPACK documentation              C
C                                                                             C
c         INTEGER M,N,IFLAG						      C
c         DOUBLE PRECISION X(N),FVEC(M)					      C
c         ----------							      C
c         CALCULATE THE FUNCTIONS AT X AND				      C
c         RETURN THIS VECTOR IN FVEC.					      C
c         ----------							      C
c         RETURN							      C
c         END								      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C	PARAMETER(MAXM=20,NPAR=3,MLWA=95)
	PARAMETER(MAXM=20)
	CHARACTER*9 FILENM1

	common/datos/FILENM1

      	DIMENSION X(NPAR),FVEC(MAXM),Swcc(MAXM,2)

	OPEN(1,FILE=FILENM1,STATUS='OLD')

C ----  -m=-(1-1/n) ----------		
	X4=-(1.D0-(1.D0/X(2)))
C ----- Calculate new Se with VG formula and find error vector FVEC-----
	READ(1,*)
	READ(1,*)
	READ(1,*)
	READ(1,*)(SWCC(1,J),J=1,2)
	WD=SWCC(1,2)-x(3)
	SWCCP=X(3)+WD*((1.D0+(X(1)*SWCC(1,1))**X(2))**X4)
	FVEC(1)=SWCCP-SWCC(1,2)
	DO 10 I=2,MPAR
		READ(1,*)(SWCC(I,J),J=1,2)
		SWCCP=X(3)+WD*((1.D0+(X(1)*SWCC(I,1))**X(2))**X4)
		FVEC(I)=SWCCP-SWCC(I,2)
10	CONTINUE
	CLOSE(1)

        RETURN
        END

        SUBROUTINE KRESULTS(MPAR,X)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C	This subroutine calculates the unsaturated conductivity curve as:     C
C                                                                             C
C	Kr(Se)=Ku/Ks = Se^0.5 [1-(1-Se^(1/m))^m]^2			      C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER(MAXM=20, NPAR=3)
      	DIMENSION X(NPAR),SWCC(MAXM,2)
	CHARACTER*9 FILENM1

	common/datos/FILENM1
 
	OPEN(3,FILE=FILENM1,STATUS='OLD')
	OPEN(7,FILE='corstat.in',STATUS='UNKNOWN')

C ---- m=1-1/n ----------		
	X4=-(1.D0-(1.D0/X(2)))
	X4P=-X4
	X4INV=1.D0/X4P
C ---- Calculate new Se with VG formula and find error vector FVEC-----
 	WRITE(*,*)'               alpha         n        Wr' 
	WRITE(*,*)' '
	WRITE(*,*)'  h(cm)         W         W_c        Error        Kr
     &      Kuns(cm/h)'
	WRITE(*,*)'-----------------------------------------------------
     &---------------'
	WRITE(7,*)' CODE    OBSERVED   PREDICTED'
	READ(3,*)
	read(3,*)
	READ(3,*)VKS
C---- First element to get Wd ---------------------------------------- 
	READ(3,*)(SWCC(1,J),J=1,2)
	WD=SWCC(1,2)-X(3)
	SWCCP=X(3)+WD*((1.D0+(X(1)*SWCC(1,1))**X(2))**X4)
	err=SWCC(1,2)-SWCCP
	SSE1=ERR*ERR
	SSE2=ERR
	SEC=(1.D0+(X(1)*SWCC(1,1))**X(2))**X4
	VGKR=DSQRT(SEC)*(1.D0-(1.D0-SEC**X4INV)**X4P)**2.D0		
	WRITE(*,100)(SWCC(1,J),J=1,2),SWCCP,ERR,VGKR,VGKR*VKS
	WRITE(7,200)'1',SWCC(1,2),SWCCP
C---- Go on with the rest ------------------------------------------
	DO 10 I=2,MPAR
		READ(3,*)(SWCC(I,J),J=1,2)
		SWCCP=x(3)+WD*((1.D0+(X(1)*SWCC(I,1))**X(2))**X4)		
		SEC=(1.D0+(X(1)*SWCC(I,1))**X(2))**X4
		VGKR=DSQRT(SEC)*(1.D0-(1.D0-SEC**X4INV)**X4P)**2.D0		
		ERR=SWCC(I,2)-SWCCP
		SSE1=SSE1+ERR*ERR
		SSE2=SSE2+ERR
		WRITE(*,100)(SWCC(I,J),J=1,2),SWCCP,ERR,VGKR,VGKR*VKS
		WRITE(7,200)'1',SWCC(I,2),SWCCP
10	CONTINUE
C--------Correlated sum of square error-----
	CSSE=SSE1-SSE2*SSE2/MPAR
C--------Root Mean Square Error (RMSE) and Mean Square Error (MSE)--
	RMSE=DSQRT(CSSE/MPAR)
	WRITE(*,*)' '
	WRITE(*,*)'Root mean sum of square errors for W(h) (RMSE)=',RMSE
	WRITE(*,*)' '
	w0_3=X(3)+WD*((1.D0+(X(1)*(1000D0/3.D0))**X(2))**X4)
	w1=X(3)+WD*((1.D0+(X(1)*1000D0)**X(2))**X4)
	w5=X(3)+WD*((1.D0+(X(1)*5000D0)**X(2))**X4)
	w15=X(3)+WD*((1.D0+(X(1)*15000D0)**X(2))**X4)
	WRITE(*,300)'1/3 bar','1 bar','5 bar','15 bar'
	WRITE(*,400)w0_3,w1,w5,w15
	WRITE(*,*)' '

	CLOSE(1)
	CLOSE(7)
100	FORMAT(F8.2,5F12.6)
200	FORMAT(A4,2F10.6)
300	FORMAT(33x,4A8)
400	FORMAT(1x,'Standard water content values:',4f8.2)
	
	RETURN
	END
	
